/*
 * MainView.java
 *
 * Created on 2008/05/24, 2:36
 */

package filesender;

import java.io.File;
import javax.swing.JFileChooser;
import javax.swing.SwingUtilities;
import javax.swing.SwingWorker;
import javax.swing.UIManager;

/**
 * 基本となるウィンドウ
 * @author  tondol
 */
public class MainView extends javax.swing.JFrame {
    private static MainView instance = null;
    private JFileChooser fc = null;
    static final int CLOSED = 0;
    static final int CONNECTED = 1;
    static final int BUSY = 2;
    
    /** Creates new form MainView */
    private MainView() {
        try {
            UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
            SwingUtilities.updateComponentTreeUI(this);
        } catch(Exception e) {
        } finally {
            initComponents();
            setState(MainView.CLOSED);
            fc = new JFileChooser();
        }
    }
    
    /**
     * 唯一のインスタンスを返す
     * @return
     */
    public static MainView getInstance() {
        if (instance == null) {
            instance = new MainView();
            javax.swing.SwingUtilities.invokeLater(new Runnable() {
                public void run() {
                    instance.setVisible(true);
                }
            });
        }
        return instance;
    }
    
    /**
     * インターフェイスの状態を変更
     * @param state
     */
    @Override
    public void setState(final int state) {
        javax.swing.SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                switch(state) {
                case CLOSED:
                    btnCRun.setEnabled(true);
                    btnSRun.setEnabled(true);
                    btnExit.setEnabled(false);
                    btnSay.setEnabled(false);
                    btnSend.setEnabled(false);
                    txtInput.setEnabled(false);
                    break;
                case CONNECTED:
                    btnCRun.setEnabled(false);
                    btnSRun.setEnabled(false);
                    btnExit.setEnabled(true);
                    btnSay.setEnabled(true);
                    btnSend.setEnabled(true);
                    txtInput.setEnabled(true);
                    break;
                case BUSY:
                    btnCRun.setEnabled(false);
                    btnSRun.setEnabled(false);
                    btnExit.setEnabled(true);
                    btnSay.setEnabled(false);
                    btnSend.setEnabled(false);
                    txtInput.setEnabled(false);
                    break;
                }
            }
        });
    }
    
    /**
     * 進捗バーの値を変更
     * @param per
     */
    public void changeProgress(final int per) {
        javax.swing.SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                progTask.setValue(per);
            }
        });
    }
    
    /**
     * メッセージを出力
     * @param msg
     */
    public void putMessage(final String msg) {
        javax.swing.SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                taOutput.append(msg + "\n");
                taOutput.setCaretPosition(taOutput.getText().length());
            }
        });
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        btnCRun = new javax.swing.JButton();
        btnSRun = new javax.swing.JButton();
        btnExit = new javax.swing.JButton();
        btnConfig = new javax.swing.JButton();
        jScrollPane1 = new javax.swing.JScrollPane();
        taOutput = new javax.swing.JTextArea();
        txtInput = new javax.swing.JTextField();
        btnSay = new javax.swing.JButton();
        progTask = new javax.swing.JProgressBar();
        btnSend = new javax.swing.JButton();

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
        setTitle("FileSender");
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                formWindowClosing(evt);
            }
        });

        btnCRun.setText("倉起動");
        btnCRun.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnCRunActionPerformed(evt);
            }
        });

        btnSRun.setText("鯖起動");
        btnSRun.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnSRunActionPerformed(evt);
            }
        });

        btnExit.setText("終了");
        btnExit.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnExitActionPerformed(evt);
            }
        });

        btnConfig.setText("設定");
        btnConfig.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnConfigActionPerformed(evt);
            }
        });

        taOutput.setColumns(20);
        taOutput.setEditable(false);
        taOutput.setFont(taOutput.getFont());
        taOutput.setLineWrap(true);
        taOutput.setRows(5);
        jScrollPane1.setViewportView(taOutput);

        txtInput.setFont(txtInput.getFont().deriveFont(txtInput.getFont().getSize()+1f));
        txtInput.setMargin(new java.awt.Insets(2, 2, 2, 2));
        txtInput.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                sendMessage(evt);
            }
        });

        btnSay.setText("発言");
        btnSay.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                sendMessage(evt);
            }
        });

        progTask.setMaximumSize(new java.awt.Dimension(32767, 18));
        progTask.setMinimumSize(new java.awt.Dimension(10, 18));
        progTask.setPreferredSize(new java.awt.Dimension(150, 18));

        btnSend.setText("送信");
        btnSend.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnSendActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jScrollPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 381, Short.MAX_VALUE)
                    .addGroup(layout.createSequentialGroup()
                        .addComponent(btnCRun)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(btnSRun)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(btnExit)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(btnConfig))
                    .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                            .addComponent(progTask, javax.swing.GroupLayout.DEFAULT_SIZE, 310, Short.MAX_VALUE)
                            .addComponent(txtInput, javax.swing.GroupLayout.DEFAULT_SIZE, 310, Short.MAX_VALUE))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                            .addComponent(btnSend)
                            .addComponent(btnSay))))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(btnCRun)
                    .addComponent(btnSRun)
                    .addComponent(btnExit)
                    .addComponent(btnConfig))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jScrollPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 211, Short.MAX_VALUE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(btnSay)
                    .addComponent(txtInput, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.CENTER)
                    .addComponent(btnSend)
                    .addComponent(progTask, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap())
        );

        pack();
        java.awt.Dimension screenSize = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
        java.awt.Dimension dialogSize = getSize();
        setLocation((screenSize.width-dialogSize.width)/2,(screenSize.height-dialogSize.height)/2);
    }// </editor-fold>//GEN-END:initComponents

    private void btnCRunActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnCRunActionPerformed
        SwingWorker worker = new SwingWorker(){
            @Override
            protected Object doInBackground() throws Exception {
                Main.runClient();
                return null;
        }}; 
        worker.execute();
}//GEN-LAST:event_btnCRunActionPerformed

    private void btnSRunActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnSRunActionPerformed
        SwingWorker worker = new SwingWorker(){
            @Override
            protected Object doInBackground() throws Exception {
                Main.runServer();
                return null;
        }}; 
        worker.execute();
        
}//GEN-LAST:event_btnSRunActionPerformed

    private void btnExitActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnExitActionPerformed
        SwingWorker worker = new SwingWorker(){
            @Override
            protected Object doInBackground() throws Exception {
                Main.closeSession();
                return null;
        }}; 
        worker.execute();
}//GEN-LAST:event_btnExitActionPerformed

    private void btnConfigActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnConfigActionPerformed
        new ConfigDialog(this, true).setVisible(true);
    }//GEN-LAST:event_btnConfigActionPerformed

    private void sendMessage(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_sendMessage
        String msg = txtInput.getText();
        if (msg.length() == 0) return;
        Main.session.sendMessage("MSG::" + msg);
        putMessage(msg);
        txtInput.setText("");
    }//GEN-LAST:event_sendMessage

    private void btnSendActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnSendActionPerformed
        // ファイル選択ダイアログを表示
        int ret = fc.showOpenDialog(this);
        if (ret != JFileChooser.APPROVE_OPTION) return;
        final File file = fc.getSelectedFile();
        // 別スレッドで送信処理
        SwingWorker worker = new SwingWorker(){
            @Override
            protected Object doInBackground() throws Exception {
                Main.sendFile(file);
                return null;
        }};
        worker.execute();
    }//GEN-LAST:event_btnSendActionPerformed

    private void formWindowClosing(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_formWindowClosing
        SwingWorker worker = new SwingWorker(){
            @Override
            protected Object doInBackground() throws Exception {
                Main.exit();
                return null;
        }};
        worker.execute();
    }//GEN-LAST:event_formWindowClosing

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton btnCRun;
    private javax.swing.JButton btnConfig;
    private javax.swing.JButton btnExit;
    private javax.swing.JButton btnSRun;
    private javax.swing.JButton btnSay;
    private javax.swing.JButton btnSend;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JProgressBar progTask;
    private javax.swing.JTextArea taOutput;
    private javax.swing.JTextField txtInput;
    // End of variables declaration//GEN-END:variables
}
